@@ -5,7 +5,7 @@
  *  This is the driver for displaying inside a window under X11,
  *  used by the graphics utility of the FreeType test suite.
  *
- *  Copyright 1999-2000, 2001, 2002, 2005 by Antoine Leca, David Turner
+ *  Copyright 1999-2002, 2005, 2006, 2013 by Antoine Leca,
  *  David Turner, Robert Wilhelm, and Werner Lemberg.
  *
  *  This file is part of the FreeType project, and may only be used
@@ -32,8 +32,11 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <sys/types.h>
+#include <unistd.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
+#include <X11/Xatom.h>
 #include <X11/cursorfont.h>
 #include <X11/keysym.h>
 
@@ -252,21 +255,21 @@ typedef  unsigned long   uint32;
 
     for ( ; h > 0; h-- )
     {
-      unsigned char*   read  = line_read;
-      unsigned short*  write = (unsigned short*)line_write;
-      int              x     = blit->width;
+      unsigned char*   lread  = line_read;
+      unsigned short*  lwrite = (unsigned short*)line_write;
+      int              x      = blit->width;
 
 
-      for ( ; x > 0; x--, read += 3, write++ )
+      for ( ; x > 0; x--, lread += 3, lwrite++ )
       {
-        unsigned int  r = read[0];
-        unsigned int  g = read[1];
-        unsigned int  b = read[2];
+        unsigned int  r = lread[0];
+        unsigned int  g = lread[1];
+        unsigned int  b = lread[2];
 
 
-        write[0] = (unsigned short)( ( ( r << 8 ) & 0xF800U ) |
-                                     ( ( g << 3 ) & 0x07E0  ) |
-                                     ( ( b >> 3 ) & 0x001F  ) );
+        lwrite[0] = (unsigned short)( ( ( r << 8 ) & 0xF800U ) |
+                                      ( ( g << 3 ) & 0x07E0  ) |
+                                      ( ( b >> 3 ) & 0x001F  ) );
       }
 
       line_read  += blit->src_pitch;
@@ -285,19 +288,19 @@ typedef  unsigned long   uint32;
 
     for ( ; h > 0; h-- )
     {
-      unsigned char*   read  = line_read;
-      unsigned short*  write = (unsigned short*)line_write;
-      int              x     = blit->width;
+      unsigned char*   lread  = line_read;
+      unsigned short*  lwrite = (unsigned short*)line_write;
+      int              x      = blit->width;
 
 
-      for ( ; x > 0; x--, read++, write++ )
+      for ( ; x > 0; x--, lread++, lwrite++ )
       {
-        unsigned int  p = read[0];
+        unsigned int  p = lread[0];
 
 
-        write[0] = (unsigned short)( ( ( p << 8 ) & 0xF800U ) |
-                                     ( ( p << 3 ) & 0x07E0  ) |
-                                     ( ( p >> 3 ) & 0x001F  ) );
+        lwrite[0] = (unsigned short)( ( ( p << 8 ) & 0xF800U ) |
+                                      ( ( p << 3 ) & 0x07E0  ) |
+                                      ( ( p >> 3 ) & 0x001F  ) );
       }
 
       line_read  += blit->src_pitch;
@@ -332,21 +335,21 @@ typedef  unsigned long   uint32;
 
     for ( ; h > 0; h-- )
     {
-      unsigned char*   read  = line_read;
-      unsigned short*  write = (unsigned short*)line_write;
-      int              x     = blit->width;
+      unsigned char*   lread  = line_read;
+      unsigned short*  lwrite = (unsigned short*)line_write;
+      int              x      = blit->width;
 
 
-      for ( ; x > 0; x--, read += 3, write++ )
+      for ( ; x > 0; x--, lread += 3, lwrite++ )
       {
-        unsigned int  r = read[0];
-        unsigned int  g = read[1];
-        unsigned int  b = read[2];
+        unsigned int  r = lread[0];
+        unsigned int  g = lread[1];
+        unsigned int  b = lread[2];
 
 
-        write[0] = (unsigned short)( ( ( b << 8 ) & 0xF800U ) |
-                                     ( ( g << 3 ) & 0x07E0  ) |
-                                     ( ( r >> 3 ) & 0x001F  ) );
+        lwrite[0] = (unsigned short)( ( ( b << 8 ) & 0xF800U ) |
+                                      ( ( g << 3 ) & 0x07E0  ) |
+                                      ( ( r >> 3 ) & 0x001F  ) );
       }
 
       line_read  += blit->src_pitch;
@@ -381,21 +384,21 @@ typedef  unsigned long   uint32;
 
     for ( ; h > 0; h-- )
     {
-      unsigned char*   read  = line_read;
-      unsigned short*  write = (unsigned short*)line_write;
-      int              x     = blit->width;
+      unsigned char*   lread  = line_read;
+      unsigned short*  lwrite = (unsigned short*)line_write;
+      int              x      = blit->width;
 
 
-      for ( ; x > 0; x--, read += 3, write++ )
+      for ( ; x > 0; x--, lread += 3, lwrite++ )
       {
-        unsigned int  r = read[0];
-        unsigned int  g = read[1];
-        unsigned int  b = read[2];
+        unsigned int  r = lread[0];
+        unsigned int  g = lread[1];
+        unsigned int  b = lread[2];
 
 
-        write[0] = (unsigned short)( ( ( r << 7 ) & 0x7C00 ) |
-                                     ( ( g << 2 ) & 0x03E0 ) |
-                                     ( ( b >> 3 ) & 0x001F ) );
+        lwrite[0] = (unsigned short)( ( ( r << 7 ) & 0x7C00 ) |
+                                      ( ( g << 2 ) & 0x03E0 ) |
+                                      ( ( b >> 3 ) & 0x001F ) );
       }
 
       line_read  += blit->src_pitch;
@@ -414,19 +417,19 @@ typedef  unsigned long   uint32;
 
     for ( ; h > 0; h-- )
     {
-      unsigned char*   read  = line_read;
-      unsigned short*  write = (unsigned short*)line_write;
-      int              x     = blit->width;
+      unsigned char*   lread  = line_read;
+      unsigned short*  lwrite = (unsigned short*)line_write;
+      int              x      = blit->width;
 
 
-      for ( ; x > 0; x--, read++, write++ )
+      for ( ; x > 0; x--, lread++, lwrite++ )
       {
-        unsigned int  p = read[0];
+        unsigned int  p = lread[0];
 
 
-        write[0] = (unsigned short)( ( ( p << 7 ) & 0x7C00 ) |
-                                     ( ( p << 2 ) & 0x03E0 ) |
-                                     ( ( p >> 3 ) & 0x001F ) );
+        lwrite[0] = (unsigned short)( ( ( p << 7 ) & 0x7C00 ) |
+                                      ( ( p << 2 ) & 0x03E0 ) |
+                                      ( ( p >> 3 ) & 0x001F ) );
       }
 
       line_read  += blit->src_pitch;
@@ -461,20 +464,21 @@ typedef  unsigned long   uint32;
 
     for ( ; h > 0; h-- )
     {
-      unsigned char*   read  = line_read;
-      unsigned short*  write = (unsigned short*)line_write;
-      int              x     = blit->width;
+      unsigned char*   lread  = line_read;
+      unsigned short*  lwrite = (unsigned short*)line_write;
+      int              x      = blit->width;
 
 
-      for ( ; x > 0; x--, read += 3, write++ )
+      for ( ; x > 0; x--, lread += 3, lwrite++ )
       {
-        unsigned int  r = read[0];
-        unsigned int  g = read[1];
-        unsigned int  b = read[2];
+        unsigned int  r = lread[0];
+        unsigned int  g = lread[1];
+        unsigned int  b = lread[2];
 
-        write[0] = (unsigned short)( ( (b << 7) & 0x7C00 ) |
-                                     ( (g << 2) & 0x03E0 ) |
-                                     ( (r >> 3) & 0x001F ) );
+
+        lwrite[0] = (unsigned short)( ( ( b << 7 ) & 0x7C00 ) |
+                                      ( ( g << 2 ) & 0x03E0 ) |
+                                      ( ( r >> 3 ) & 0x001F ) );
       }
 
       line_read  += blit->src_pitch;
@@ -526,19 +530,19 @@ typedef  unsigned long   uint32;
 
     for ( ; h > 0; h-- )
     {
-      unsigned char*   read  = line_read;
-      unsigned char*   write = line_write;
-      int              x     = blit->width;
+      unsigned char*   lread  = line_read;
+      unsigned char*   lwrite = line_write;
+      int              x      = blit->width;
 
 
-      for ( ; x > 0; x--, read++, write += 3 )
+      for ( ; x > 0; x--, lread++, lwrite += 3 )
       {
-        unsigned char  p = read[0];
+        unsigned char  p = lread[0];
 
 
-        write[0] = p;
-        write[1] = p;
-        write[2] = p;
+        lwrite[0] = p;
+        lwrite[1] = p;
+        lwrite[2] = p;
       }
 
       line_read  += blit->src_pitch;
@@ -573,16 +577,16 @@ typedef  unsigned long   uint32;
 
     for ( ; h > 0; h-- )
     {
-      unsigned char*   read  = line_read;
-      unsigned char*   write = line_write;
-      int              x     = blit->width;
+      unsigned char*   lread  = line_read;
+      unsigned char*   lwrite = line_write;
+      int              x      = blit->width;
 
 
-      for ( ; x > 0; x--, read += 3, write += 3 )
+      for ( ; x > 0; x--, lread += 3, lwrite += 3 )
       {
-        write[0] = read[2];
-        write[1] = read[1];
-        write[2] = read[0];
+        lwrite[0] = lread[2];
+        lwrite[1] = lread[1];
+        lwrite[2] = lread[0];
       }
 
       line_read  += blit->src_pitch;
@@ -617,18 +621,21 @@ typedef  unsigned long   uint32;
 
     for ( ; h > 0; h-- )
     {
-      unsigned char*   read  = line_read;
-      unsigned char*   write = line_write;
-      int              x     = blit->width;
+      unsigned char*   lread  = line_read;
+      unsigned char*   lwrite = line_write;
+      int              x      = blit->width;
 
 
-      for ( ; x > 0; x--, read += 3, write += 4 )
+      for ( ; x > 0; x--, lread += 3, lwrite += 4 )
       {
-        uint32  r = read[0];
-        uint32  g = read[1];
-        uint32  b = read[2];
+        uint32  r = lread[0];
+        uint32  g = lread[1];
+        uint32  b = lread[2];
 
-        *(uint32*)write = ( ( r << 24 ) | ( g << 16 ) | ( b << 8 ) );
+
+        *(uint32*)lwrite = ( r << 24 ) |
+                           ( g << 16 ) |
+                           ( b <<  8 );
       }
 
       line_read  += blit->src_pitch;
@@ -647,17 +654,19 @@ typedef  unsigned long   uint32;
 
     for ( ; h > 0; h-- )
     {
-      unsigned char*  read  = line_read;
-      unsigned char*  write = line_write;
-      int             x     = blit->width;
+      unsigned char*  lread  = line_read;
+      unsigned char*  lwrite = line_write;
+      int             x      = blit->width;
 
 
-      for ( ; x > 0; x--, read ++, write += 4 )
+      for ( ; x > 0; x--, lread ++, lwrite += 4 )
       {
-        uint32  p = read[0];
+        uint32  p = lread[0];
 
 
-        *(uint32*)write = ( ( p << 24 ) | ( p << 16 ) | ( p << 8 ) );
+        *(uint32*)lwrite = ( p << 24 ) |
+                           ( p << 16 ) |
+                           ( p <<  8 );
       }
 
       line_read  += blit->src_pitch;
@@ -692,18 +701,21 @@ typedef  unsigned long   uint32;
 
     for ( ; h > 0; h-- )
     {
-      unsigned char*  read  = line_read;
-      unsigned char*  write = line_write;
-      int             x     = blit->width;
+      unsigned char*  lread  = line_read;
+      unsigned char*  lwrite = line_write;
+      int             x      = blit->width;
 
 
-      for ( ; x > 0; x--, read += 3, write += 4 )
+      for ( ; x > 0; x--, lread += 3, lwrite += 4 )
       {
-        uint32  r = read[0];
-        uint32  g = read[1];
-        uint32  b = read[2];
+        uint32  r = lread[0];
+        uint32  g = lread[1];
+        uint32  b = lread[2];
 
-        *(uint32*)write = ( ( r << 16 ) | ( g << 8 ) | ( b << 0 ) );
+
+        *(uint32*)lwrite = ( r << 16 ) |
+                           ( g <<  8 ) |
+                           ( b <<  0 );
       }
 
       line_read  += blit->src_pitch;
@@ -722,17 +734,19 @@ typedef  unsigned long   uint32;
 
     for ( ; h > 0; h-- )
     {
-      unsigned char*  read  = line_read;
-      unsigned char*  write = line_write;
-      int             x     = blit->width;
+      unsigned char*  lread  = line_read;
+      unsigned char*  lwrite = line_write;
+      int             x      = blit->width;
 
 
-      for ( ; x > 0; x--, read ++, write += 4 )
+      for ( ; x > 0; x--, lread ++, lwrite += 4 )
       {
-        uint32  p = read[0];
+        uint32  p = lread[0];
 
 
-        *(uint32*)write = ( ( p << 16 ) | ( p << 8 ) | ( p << 0 ) );
+        *(uint32*)lwrite = ( p << 16 ) |
+                           ( p <<  8 ) |
+                           ( p <<  0 );
       }
 
       line_read  += blit->src_pitch;
@@ -767,18 +781,21 @@ typedef  unsigned long   uint32;
 
     for ( ; h > 0; h-- )
     {
-      unsigned char*  read  = line_read;
-      unsigned char*  write = line_write;
-      int             x     = blit->width;
+      unsigned char*  lread  = line_read;
+      unsigned char*  lwrite = line_write;
+      int             x      = blit->width;
 
 
-      for ( ; x > 0; x--, read += 3, write += 4 )
+      for ( ; x > 0; x--, lread += 3, lwrite += 4 )
       {
-        uint32  r = read[0];
-        uint32  g = read[1];
-        uint32  b = read[2];
+        uint32  r = lread[0];
+        uint32  g = lread[1];
+        uint32  b = lread[2];
 
-        *(uint32*)write = ( ( r << 8 ) | ( g << 16 ) | ( b << 24 ) );
+
+        *(uint32*)lwrite = ( r <<  8 ) |
+                           ( g << 16 ) |
+                           ( b << 24 );
       }
 
       line_read  += blit->src_pitch;
@@ -813,18 +830,21 @@ typedef  unsigned long   uint32;
 
     for ( ; h > 0; h-- )
     {
-      unsigned char*  read  = line_read;
-      unsigned char*  write = line_write;
-      int             x     = blit->width;
+      unsigned char*  lread  = line_read;
+      unsigned char*  lwrite = line_write;
+      int             x      = blit->width;
 
 
-      for ( ; x > 0; x--, read += 3, write += 4 )
+      for ( ; x > 0; x--, lread += 3, lwrite += 4 )
       {
-        uint32  r = read[0];
-        uint32  g = read[1];
-        uint32  b = read[2];
+        uint32  r = lread[0];
+        uint32  g = lread[1];
+        uint32  b = lread[2];
 
-        *(uint32*)write = ( ( r << 0 ) | ( g << 8 ) | ( b << 16 ) );
+
+        *(uint32*)lwrite = ( r <<  0 ) |
+                           ( g <<  8 ) |
+                           ( b << 16 );
       }
 
       line_read  += blit->src_pitch;
@@ -861,8 +881,7 @@ typedef  unsigned long   uint32;
     &gr_x11_format_bgr0888,
     &gr_x11_format_rgb8880,
     &gr_x11_format_bgr8880,
-
-    NULL,
+    NULL
   };
 
   typedef struct  grX11DeviceRec_
@@ -917,8 +936,8 @@ typedef  unsigned long   uint32;
       XDepth*      formats;
       XVisualInfo  templ;
 
-
-      formats = XListPixmapFormats( x11dev.display, &count );
+      templ.screen = DefaultScreen( x11dev.display );
+      formats      = XListPixmapFormats( x11dev.display, &count );
 
 #ifdef TEST
       printf( "available pixmap formats\n" );
@@ -946,7 +965,6 @@ typedef  unsigned long   uint32;
             XVisualInfo*  visual;
             XVisualInfo*  visuals;
 
-            templ.screen = DefaultScreen( x11dev.display );
             templ.depth  = format->depth;
             visuals      = XGetVisualInfo( x11dev.display,
                                            VisualScreenMask | VisualDepthMask,
@@ -956,29 +974,30 @@ typedef  unsigned long   uint32;
             for ( visual = visuals; count2 > 0; count2--, visual++ )
             {
 #ifdef TEST
-              const char*  string = "unknown";
-
+              const char*  visualClass;
 
               switch ( visual->Class )
               {
               case TrueColor:
-                string = "TrueColor";
+                visualClass = "TrueColor";
                 break;
               case DirectColor:
-                string = "DirectColor";
+                visualClass = "DirectColor";
                 break;
               case PseudoColor:
-                string = "PseudoColor";
+                visualClass = "PseudoColor";
                 break;
               case StaticGray:
-                string = "StaticGray";
+                visualClass = "StaticGray";
                 break;
               case StaticColor:
-                string = "StaticColor";
+                visualClass = "StaticColor";
                 break;
               case GrayScale:
-                string = "GrayScale";
+                visualClass = "GrayScale";
                 break;
+              default:
+                visualClass = "unknown";
               }
 
               printf( ">   RGB %04lx:%04lx:%04lx, colors %3d, bits %2d  %s\n",
@@ -987,7 +1006,7 @@ typedef  unsigned long   uint32;
                       visual->blue_mask,
                       visual->colormap_size,
                       visual->bits_per_rgb,
-                      string );
+                      visualClass );
 #endif /* TEST */
 
               /* compare to the list of supported formats */
@@ -1350,7 +1369,11 @@ typedef  unsigned long   uint32;
       XSetWindowAttributes  xswa;
       long                  xswa_mask = CWBackPixel | CWEventMask | CWCursor;
 
-      xswa.border_pixel = BlackPixel( display, screen);
+      pid_t                 pid;
+      Atom                  _NET_WM_PID;
+
+
+      xswa.border_pixel = BlackPixel( display, screen );
 
       if (surface->visual == DefaultVisual( display, screen ) )
       {
@@ -1409,6 +1432,12 @@ typedef  unsigned long   uint32;
 
       XSetWMProperties( display, surface->win, &xtp, &xtp,
                         NULL, 0, &xsh, NULL, NULL );
+
+      pid = getpid();
+      _NET_WM_PID = XInternAtom( display, "_NET_WM_PID", False );
+      XChangeProperty( display, surface->win, _NET_WM_PID,
+                       XA_CARDINAL, 32, PropModeReplace,
+                       (unsigned char *)&pid, 1 );
     }
 
     surface->root.done         = (grDoneSurfaceFunc)gr_x11_surface_done;
@@ -1522,7 +1551,7 @@ typedef  unsigned long   uint32;
         if ( event.key == grKeyEsc )
           return 0;
 
-        /* otherwise, display key string */
+        /* otherwise, display key name */
         color.value = ( color.value + 8 ) & 127;
         {
           int          count = sizeof ( key_names ) / sizeof ( key_names[0] );
