@@ -192,6 +192,7 @@
     int             autohint;          /* force auto-hinting          */
     int             lcd_mode;
     int             preload;           /* force font file preloading  */
+    int             color;             /* load color bitmaps          */
 
     /* don't touch the following fields! */
 
@@ -218,7 +219,8 @@
   /* install a font */
   FT_Error
   FTDemo_Install_Font( FTDemo_Handle*  handle,
-                       const char*     filepath );
+                       const char*     filepath,
+                       FT_Bool         outline_only );
 
 
   void
