@@ -2,7 +2,7 @@
 /*                                                                          */
 /*  The FreeType project -- a free and portable quality TrueType renderer.  */
 /*                                                                          */
-/*  Copyright 1996-2000, 2003, 2004, 2005 by                                */
+/*  Copyright 1996-2000, 2003-2005, 2010-2013 by                            */
 /*  D. Turner, R.Wilhelm, and W. Lemberg                                    */
 /*                                                                          */
 /*                                                                          */
@@ -26,15 +26,15 @@
 #include "graph.h"
 #include "grfont.h"
 
-#define  DIM_X   500
-#define  DIM_Y   400
+#define  DIM_X   640
+#define  DIM_Y   480
 
 #define  CENTER_X   ( bit.width / 2 )
 #define  CENTER_Y   ( bit.rows / 2 )
 
 #define  MAXPTSIZE  500                 /* dtp */
 
-  char  Header[128];
+  char  Header[256];
   char* new_header = 0;
 
   const unsigned char*  Text = (unsigned char*)
@@ -55,13 +55,15 @@
   grSurface*    surface;      /* current display surface     */
   grBitmap      bit;          /* current display bitmap      */
 
-  int  num_glyphs;            /* number of glyphs */
-  int  ptsize;                /* current point size */
+  int  width     = DIM_X;     /* window width                */
+  int  height    = DIM_Y;     /* window height               */
+
+  int  num_glyphs;            /* number of glyphs            */
+  int  ptsize;                /* current point size          */
 
   int  hinted    = 1;         /* is glyph hinting active?    */
   int  antialias = 1;         /* is anti-aliasing active?    */
   int  use_sbits = 1;         /* do we use embedded bitmaps? */
-  int  low_prec  = 0;         /* force low precision         */
   int  Num;                   /* current first glyph index   */
 
   int  res       = 72;
@@ -168,8 +170,8 @@
     grInitDevices();
 
     bit.mode  = gr_pixel_mode_gray;
-    bit.width = DIM_X;
-    bit.rows  = DIM_Y;
+    bit.width = width;
+    bit.rows  = height;
     bit.grays = 256;
 
     surface = grNewSurface( 0, &bit );
@@ -245,7 +247,7 @@
     int  flags;
 
 
-    flags = FT_LOAD_DEFAULT | FT_LOAD_IGNORE_GLOBAL_ADVANCE_WIDTH;
+    flags = FT_LOAD_DEFAULT;
 
     if ( !hint )
       flags |= FT_LOAD_NO_HINTING;
@@ -261,14 +263,13 @@
   Render_All( int  first_glyph,
               int  pt_size )
   {
-    FT_F26Dot6  start_x, start_y, step_x, step_y, x, y;
+    FT_F26Dot6  start_x, start_y, step_y, x, y;
     int         i;
 
 
     start_x = 4;
     start_y = 36 + pt_size;
 
-    step_x = size->metrics.x_ppem + 4;
     step_y = size->metrics.y_ppem + 10;
 
     x = start_x;
@@ -323,7 +324,7 @@
   static FT_Error
   Render_Text( int  first_glyph )
   {
-    FT_F26Dot6  start_x, start_y, step_x, step_y, x, y;
+    FT_F26Dot6  start_x, start_y, step_y, x, y;
     int         i;
 
     const unsigned char*  p;
@@ -332,7 +333,6 @@
     start_x = 4;
     start_y = 32 + size->metrics.y_ppem;
 
-    step_x = size->metrics.x_ppem + 4;
     step_y = size->metrics.y_ppem + 10;
 
     x = start_x;
@@ -396,7 +396,7 @@
 
 
     Clear_Display();
-
+    grSetLineHeight( 10 );
     grGotoxy( 0, 0 );
     grSetMargin( 2, 1 );
     grGotobitmap( &bit );
@@ -412,7 +412,6 @@
     grWriteln( "  a         : toggle anti-aliasing" );
     grWriteln( "  h         : toggle outline hinting" );
     grWriteln( "  b         : toggle embedded bitmaps" );
-    grWriteln( "  l         : toggle low precision rendering" );
     grWriteln( "  space     : toggle rendering mode" );
     grLn();
     grWriteln( "  n         : next font" );
@@ -480,13 +479,6 @@
     case grKEY( 'p' ):
       return (int)event->key;
 
-    case grKEY( 'l' ):
-      low_prec = !low_prec;
-      new_header = low_prec
-                     ? (char *)"rendering precision is now forced to low"
-                     : (char *)"rendering precision is now normal";
-      break;
-
     case grKEY( 'h' ):
       hinted = !hinted;
       new_header = hinted ? (char *)"glyph hinting is now active"
@@ -617,19 +609,39 @@
   static void
   usage( char*  execname )
   {
-    fprintf( stderr,  "\n" );
-    fprintf( stderr,  "ftmulti: multiple masters font viewer - part of FreeType\n" );
-    fprintf( stderr,  "--------------------------------------------------------\n" );
-    fprintf( stderr,  "\n" );
-    fprintf( stderr,  "Usage: %s [options below] ppem fontname[.pfb|.ttf] ...\n",
+    fprintf( stderr,
+      "\n"
+      "ftmulti: multiple masters font viewer - part of FreeType\n"
+      "--------------------------------------------------------\n"
+      "\n" );
+    fprintf( stderr,
+      "Usage: %s [options] pt font ...\n"
+      "\n",
              execname );
-    fprintf( stderr,  "\n" );
-    fprintf( stderr,  "  -e encoding  select encoding (default: no encoding)\n" );
-    fprintf( stderr,  "  -r R         use resolution R dpi (default: 72 dpi)\n" );
-    fprintf( stderr,  "  -f index     specify first glyph index to display\n" );
-    fprintf( stderr,  "  -d \"axis1 axis2 ...\"\n"
-                      "               specify the design coordinates for each axis\n" );
-    fprintf( stderr,  "\n" );
+    fprintf( stderr,
+      "  pt           The point size for the given resolution.\n"
+      "               If resolution is 72dpi, this directly gives the\n"
+      "               ppem value (pixels per EM).\n" );
+    fprintf( stderr,
+      "  font         The font file(s) to display.\n"
+      "\n" );
+    fprintf( stderr,
+      "  -w W         Set window width to W pixels (default: %dpx).\n"
+      "  -h H         Set window height to H pixels (default: %dpx).\n"
+      "\n",
+             DIM_X, DIM_Y );
+    fprintf( stderr,
+      "  -e encoding  Specify encoding tag (default: no encoding).\n"
+      "               Common values: `unic' (Unicode), `symb' (symbol),\n"
+      "               `ADOB' (Adobe standard), `ADBC' (Adobe custom).\n"
+      "  -r R         Use resolution R dpi (default: 72dpi).\n"
+      "  -f index     Specify first glyph index to display.\n"
+      "  -d \"axis1 axis2 ...\"\n"
+      "               Specify the design coordinates for each\n"
+      "               Multiple Master axis at start-up.\n"
+      "\n"
+      "  -v           Show version."
+      "\n" );
 
     exit( 1 );
   }
@@ -651,9 +663,14 @@
 
     execname = ft_basename( argv[0] );
 
+    /* Initialize engine */
+    error = FT_Init_FreeType( &library );
+    if ( error )
+      PanicZ( "Could not initialize FreeType library" );
+
     while ( 1 )
     {
-      option = getopt( argc, argv, "d:e:f:r:" );
+      option = getopt( argc, argv, "d:e:f:h:r:vw:" );
 
       if ( option == -1 )
         break;
@@ -672,12 +689,39 @@
         first_glyph = atoi( optarg );
         break;
 
+      case 'h':
+        height = atoi( optarg );
+        if ( height < 1 )
+          usage( execname );
+        break;
+
       case 'r':
         res = atoi( optarg );
         if ( res < 1 )
           usage( execname );
         break;
 
+      case 'v':
+        {
+          FT_Int  major, minor, patch;
+
+
+          FT_Library_Version( library, &major, &minor, &patch );
+
+          printf( "ftmulti (FreeType) %d.%d", major, minor );
+          if ( patch )
+            printf( ".%d", patch );
+          printf( "\n" );
+          exit( 0 );
+        }
+        break;
+
+      case 'w':
+        width = atoi( optarg );
+        if ( width < 1 )
+          usage( execname );
+        break;
+
       default:
         usage( execname );
         break;
@@ -695,11 +739,6 @@
 
     file = 1;
 
-    /* Initialize engine */
-    error = FT_Init_FreeType( &library );
-    if ( error )
-      PanicZ( "Could not initialize FreeType library" );
-
   NewFile:
     ptsize      = orig_ptsize;
     hinted      = 1;
@@ -795,7 +834,7 @@
           Render_All( Num, ptsize );
         }
 
-        sprintf( Header, "%s %s (file %s)",
+        sprintf( Header, "%.50s %.50s (file %.100s)",
                          face->family_name,
                          face->style_name,
                          ft_basename( argv[file] ) );
@@ -813,13 +852,14 @@
 
           for ( n = 0; n < (int)multimaster->num_axis; n++ )
           {
-            char  temp[32];
+            char  temp[100];
 
 
-            sprintf( temp, "  %s:%g",
+            sprintf( temp, "  %.50s:%g",
                            multimaster->axis[n].name,
-                           design_pos[n]/65536. );
-            strcat( Header, temp );
+                           design_pos[n] / 65536.0 );
+            strncat( Header, temp,
+                     sizeof ( Header ) - strlen( Header ) - 1 );
           }
         }
         grWriteCellString( &bit, 0, 16, Header, fore_color );
@@ -830,7 +870,7 @@
       }
       else
       {
-        sprintf( Header, "%s: not an MM font file, or could not be opened",
+        sprintf( Header, "%.100s: not an MM font file, or could not be opened",
                          ft_basename( argv[file] ) );
       }
 
