@@ -33,6 +33,7 @@
 #include FT_SYNTHESIS_H
 #include FT_LCD_FILTER_H
 #include FT_CFF_DRIVER_H
+#include FT_TRUETYPE_DRIVER_H
 
 
 #define MAXPTSIZE  500                 /* dtp */
@@ -66,7 +67,7 @@
 #endif
 
 
-#define N_HINTING_ENGINES  2
+#define N_CFF_HINTING_ENGINES  2
 
 
   enum
@@ -98,7 +99,8 @@
     double         radius;
     double         slant;
 
-    int            hinting_engine;    /* for CFF */
+    int            cff_hinting_engine;
+    int            tt_interpreter_version;
 
     int            font_idx;
     int            offset;            /* as selected by the user */
@@ -113,7 +115,7 @@
   } status = { 1,
                DIM_X, DIM_Y, RENDER_MODE_ALL, FT_ENCODING_NONE,
                72, 48, -1, 1.0, 0.04, 0.04, 0.02, 0.22,
-               FT_CFF_HINTING_FREETYPE,
+               0, 0, /* default values are set at runtime */
                0, 0, 0, 0, 0,
                0, { 0x10, 0x40, 0x70, 0x40, 0x10 }, 2 };
 
@@ -678,7 +680,8 @@
         }
       }
 
-      snprintf( text, 256, "%g: %n%s", pt_size / 64.0, &start, p );
+      start = snprintf( text, 256, "%g: ", pt_size / 64.0 );
+      snprintf( text + start, 256 - start, "%s", p );
 
       p    = text;
       pEnd = p + strlen( text );
@@ -769,25 +772,26 @@
     grWriteln( "  backspace cycle backwards               l         cycle backwards         " );
     grWriteln( "                                                                            " );
     grWriteln( "b           toggle embedded bitmaps     x, X        adjust horizontal       " );
-    grWriteln( "c           toggle cache modes                       emboldening (in mode 2)" );
-    grWriteln( "                                        y, Y        adjust vertical         " );
-    grWriteln( "p, n        previous/next font                       emboldening (in mode 2)" );
-    grWriteln( "                                        s, S        adjust slanting         " );
-    grWriteln( "Up, Down    adjust size by 1 unit                    (in mode 3)            " );
-    grWriteln( "PgUp, PgDn  adjust size by 10 units     r, R        adjust stroking radius  " );
-    grWriteln( "                                                     (in mode 4)            " );
-    grWriteln( "Left, Right adjust index by 1                                               " );
-    grWriteln( "F7, F8      adjust index by 10          L           toggle custom           " );
-    grWriteln( "F9, F10     adjust index by 100                      LCD filtering          " );
-    grWriteln( "F11, F12    adjust index by 1000        [, ]        select custom LCD       " );
-    grWriteln( "                                                      filter weight         " );
-    grWriteln( "h           toggle hinting                            (if custom filtering) " );
-    grWriteln( "H           cycle through hinting       -, +(=)     adjust selected custom  " );
-    grWriteln( "             engines (if available)                  LCD filter weight      " );
-    grWriteln( "f           toggle forced auto-                                             " );
-    grWriteln( "             hinting (if hinting)       G           show gamma ramp         " );
-    grWriteln( "                                        g, v        adjust gamma value      " );
-    grWriteln( "a           toggle anti-aliasing                                            " );
+    grWriteln( "c           toggle color glyphs                      emboldening (in mode 2)" );
+    grWriteln( "K           toggle cache modes          y, Y        adjust vertical         " );
+    grWriteln( "                                                     emboldening (in mode 2)" );
+    grWriteln( "p, n        previous/next font          s, S        adjust slanting         " );
+    grWriteln( "                                                     (in mode 3)            " );
+    grWriteln( "Up, Down    adjust size by 1 unit       r, R        adjust stroking radius  " );
+    grWriteln( "PgUp, PgDn  adjust size by 10 units                  (in mode 4)            " );
+    grWriteln( "                                                                            " );
+    grWriteln( "Left, Right adjust index by 1           L           toggle custom           " );
+    grWriteln( "F7, F8      adjust index by 10                       LCD filtering          " );
+    grWriteln( "F9, F10     adjust index by 100         [, ]        select custom LCD       " );
+    grWriteln( "F11, F12    adjust index by 1000                      filter weight         " );
+    grWriteln( "                                                      (if custom filtering) " );
+    grWriteln( "h           toggle hinting              -, +(=)     adjust selected custom  " );
+    grWriteln( "H           cycle through hinting                    LCD filter weight      " );
+    grWriteln( "             engines (if available)                                         " );
+    grWriteln( "f           toggle forced auto-         G           show gamma ramp         " );
+    grWriteln( "             hinting (if hinting)       g, v        adjust gamma value      " );
+    grWriteln( "                                                                            " );
+    grWriteln( "a           toggle anti-aliasing        q, ESC      quit ftview             " );
     /*          |----------------------------------|    |----------------------------------| */
     grLn();
     grLn();
@@ -866,20 +870,64 @@
   }
 
 
-  static void
-  event_hinting_engine_change( int  delta )
+  static int
+  event_cff_hinting_engine_change( int  delta )
   {
+    int  new_cff_hinting_engine;
+
+
     if ( delta )
-      status.hinting_engine = ( status.hinting_engine +
-                                delta                 +
-                                N_HINTING_ENGINES     ) % N_HINTING_ENGINES;
+      new_cff_hinting_engine =
+        ( status.cff_hinting_engine +
+          delta                     +
+          N_CFF_HINTING_ENGINES     ) % N_CFF_HINTING_ENGINES;
 
-    FT_Property_Set( handle->library,
-                     "cff",
-                     "hinting-engine", &status.hinting_engine );
+    error = FT_Property_Set( handle->library,
+                             "cff",
+                             "hinting-engine",
+                             &new_cff_hinting_engine );
 
-    FTC_Manager_RemoveFaceID( handle->cache_manager,
-                              handle->scaler.face_id );
+    if ( !error )
+    {
+      /* Resetting the cache is perhaps a bit harsh, but I'm too  */
+      /* lazy to walk over all loaded fonts to check whether they */
+      /* are of type CFF, then unloading them explicitly.         */
+      FTC_Manager_Reset( handle->cache_manager );
+      status.cff_hinting_engine = new_cff_hinting_engine;
+      return 1;
+    }
+
+    return 0;
+  }
+
+
+  static int
+  event_tt_interpreter_version_change( void )
+  {
+    FT_UInt  new_interpreter_version;
+
+
+    if ( status.tt_interpreter_version == TT_INTERPRETER_VERSION_35 )
+      new_interpreter_version = TT_INTERPRETER_VERSION_38;
+    else
+      new_interpreter_version = TT_INTERPRETER_VERSION_35;
+
+    error = FT_Property_Set( handle->library,
+                             "truetype",
+                             "interpreter-version",
+                             &new_interpreter_version );
+
+    if ( !error )
+    {
+      /* Resetting the cache is perhaps a bit harsh, but I'm too  */
+      /* lazy to walk over all loaded fonts to check whether they */
+      /* are of type TTF, then unloading them explicitly.         */
+      FTC_Manager_Reset( handle->cache_manager );
+      status.tt_interpreter_version = new_interpreter_version;
+      return 1;
+    }
+
+    return 0;
   }
 
 
@@ -1086,6 +1134,12 @@
       break;
 
     case grKEY( 'c' ):
+      handle->color = !handle->color;
+      FTDemo_Update_Current_Flags( handle );
+      status.update = 1;
+      break;
+
+    case grKEY( 'K' ):
       handle->use_sbits_cache = !handle->use_sbits_cache;
       status.update = 1;
       break;
@@ -1118,11 +1172,11 @@
         if ( !error )
         {
           module = &face->driver->root;
+
           if ( !strcmp( module->clazz->module_name, "cff" ) )
-          {
-            event_hinting_engine_change( 1 );
-            status.update = 1;
-          }
+            status.update = event_cff_hinting_engine_change( 1 );
+          else if ( !strcmp( module->clazz->module_name, "truetype" ) )
+            status.update = event_tt_interpreter_version_change();
         }
       }
       break;
@@ -1420,7 +1474,7 @@
     if ( !( status.render_mode == RENDER_MODE_TEXT      ||
             status.render_mode == RENDER_MODE_WATERFALL ) )
     {
-      const char*  encoding;
+      const char*  encoding = NULL;
 
 
       switch ( status.encoding )
@@ -1489,7 +1543,7 @@
 
     /* render mode */
     {
-      const char*  render_mode;
+      const char*  render_mode = NULL;
 
 
       switch ( status.render_mode )
@@ -1563,7 +1617,7 @@
 
     if ( handle->antialias )
     {
-      const char*  lcd_mode;
+      const char*  lcd_mode = NULL;
 
 
       switch ( handle->lcd_mode )
@@ -1612,13 +1666,13 @@
     {
       /* hinting engine */
 
-      FT_Module    module = &face->driver->root;
-      const char*  hinting_engine;
+      FT_Module    module         = &face->driver->root;
+      const char*  hinting_engine = NULL;
 
 
       if ( !strcmp( module->clazz->module_name, "cff" ) )
       {
-        switch ( status.hinting_engine )
+        switch ( status.cff_hinting_engine )
         {
         case FT_CFF_HINTING_FREETYPE:
           hinting_engine = "FreeType";
@@ -1627,7 +1681,23 @@
           hinting_engine = "Adobe";
           break;
         }
+      }
 
+      else if ( !strcmp( module->clazz->module_name, "truetype" ) )
+      {
+        switch ( status.tt_interpreter_version )
+        {
+        case TT_INTERPRETER_VERSION_35:
+          hinting_engine = "v35";
+          break;
+        case TT_INTERPRETER_VERSION_38:
+          hinting_engine = "v38";
+          break;
+        }
+      }
+
+      if ( hinting_engine )
+      {
         sprintf( buf, "engine: %s",
                       hinting_engine );
         grWriteCellString( display->bitmap, 0, (line++) * HEADER_HEIGHT,
@@ -1643,6 +1713,12 @@
     grWriteCellString( display->bitmap, 0, (line++) * HEADER_HEIGHT,
                        buf, display->fore_color );
 
+    /* embedded color bitmaps */
+    sprintf( buf, "color bitmaps: %s",
+                  handle->color ? "on" : "off" );
+    grWriteCellString( display->bitmap, 0, (line++) * HEADER_HEIGHT,
+                       buf, display->fore_color );
+
     /* cache */
     sprintf( buf, "cache: %s",
                   handle->use_sbits_cache ? "on" : "off" );
@@ -1848,9 +1924,14 @@
     parse_cmdline( &argc, &argv );
 
     FT_Library_SetLcdFilter( handle->library, FT_LCD_FILTER_DEFAULT );
-    FT_Property_Set( handle->library,
+
+    /* get the default value as compiled into FreeType */
+    FT_Property_Get( handle->library,
                      "cff",
-                     "hinting-engine", &status.hinting_engine );
+                     "hinting-engine", &status.cff_hinting_engine );
+    FT_Property_Get( handle->library,
+                     "truetype",
+                     "interpreter-version", &status.tt_interpreter_version );
 
     handle->encoding = status.encoding;
 
@@ -1858,7 +1939,7 @@
       FTDemo_Set_Preload( handle, 1 );
 
     for ( ; argc > 0; argc--, argv++ )
-      FTDemo_Install_Font( handle, argv[0] );
+      FTDemo_Install_Font( handle, argv[0], 0 );
 
     if ( handle->num_fonts == 0 )
       Fatal( "could not find/open any font file" );
